package com.zfabrik.impl.servletjsp.webapp;

import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Logger;
import java.util.regex.Pattern;

import org.eclipse.jetty.util.PatternMatcher;
import org.eclipse.jetty.util.resource.Resource;
import org.eclipse.jetty.webapp.Descriptor;
import org.eclipse.jetty.webapp.DescriptorProcessor;
import org.eclipse.jetty.webapp.WebAppContext;
import org.eclipse.jetty.webapp.WebDescriptor;
import org.eclipse.jetty.webapp.WebInfConfiguration;

/**
 * Helper for Jetty 9 Fragment loading (#2109)
 */
public class Jetty9FragmentLoader {
	private final static Logger LOG = Logger.getLogger(Jetty9FragmentLoader.class.getName());

    /**
     *  Load web fragments from the class path
     */
    public static void loadWebFragments(WebAppContext wac) throws IOException {
    	wac.getMetaData().addDescriptorProcessor(new DescriptorProcessor() {
    		@Override
    		public void process(WebAppContext context, Descriptor descriptor) throws Exception {
    			if (descriptor instanceof WebDescriptor) {
    				if (context.getAttribute(Jetty9FragmentLoader.class.getName())==null) {
    					if (context.getMetaData().getOrdering()==null) {
	    					// Our one run for this context:
	    					// Check for fragment providing libraries and add them to WebInfJars.
	    					// This happens after all scanning has occured and Jetty 
	    					// processes fragments.
	    					// NOTE: This only applies when there is no declared ordering!
    						Enumeration<URL> urls = wac.getClassLoader().getResources("META-INF/web-fragment.xml");
    						List<URI> uris = new ArrayList<URI>(); 
    						while (urls.hasMoreElements()) {
    							// We want the jar file only
    							uris.add(new URI(stripJarFile(urls.nextElement()))); 
							}
    						// Mimick the jar filtering of jetty
    				        String spattern = (String)context.getAttribute(WebInfConfiguration.CONTAINER_JAR_PATTERN);
    				        Pattern pattern = (spattern== null ? null : Pattern.compile(spattern));
    				        PatternMatcher matcher = new PatternMatcher() {
    				        	public void matched(URI uri) throws Exception {
    				        		Resource key = Resource.newResource(uri);
    				        		LOG.warning(()->"Jetty9-Fragment Patch: Adding additional jar "+key); 
    				        		wac.getMetaData().addWebInfJar(key);
    				        	};
    				        };
    				        matcher.match(pattern, uris.toArray(new URI[uris.size()]), false);
    				        // done
	    					wac.setAttribute(Jetty9FragmentLoader.class.getName(),Boolean.TRUE);
    					}
    				}
    			}
    		}
    	});
    }

    private static String stripJarFile(URL url) throws IOException {
        String path = url.toExternalForm();
        if (path.startsWith("jar:")) {
            return removeStart(substringBefore(path, "!/"), "jar:");
        } else if (path.startsWith("file:")) {
            return substringBefore(path, "/META-INF");
        }
        throw new IllegalArgumentException("Unsupported URL: " + path);
    }
    
    private  static String removeStart(String path, String start) {
    	if (path!=null && path.startsWith(start)) {
    		return path.substring(start.length());
    	} else {
    		return path;
    	}
	}

	private  static String substringBefore(String path, String separation) {
        if (path==null || separation==null) {
            return path;
        }
        int p = path.lastIndexOf(separation);
        if (p<0) {
            return path;
        }
        return path.substring(0, p);
	}

 }
